#include <stdio.h>
#include "es.H"
#include "ind_global.H"
#include "ind_uniform.H"
#include "ind_separate.H"
#include "myrandom.h"


es::es(int dim, fitnessfct *fct, int mu, int lambda, 
	    double delta, int generations, int whichmut) {
  int i;

  /* Kopieren der Argumente */
  the_fitnessfct = fct;
  the_mu = mu;
  the_lambda = lambda;
  the_delta = delta;
  the_generations = generations;

  /* Anlegen der Populationen */
  parents = new (ind*)[the_mu];
  kids = new (ind*)[the_lambda];
  
  /* Initialisieren der Eltern */
  for (i=0; i<the_mu; i++) {
    switch (whichmut) {
    case 0: parents[i] = new ind_global(dim,fct);
      break;
    case 1: parents[i] = new ind_uniform(dim,fct);
            (parents[i])->setdelta(the_delta);
      break;
    case 2: parents[i] = new ind_separate(dim,fct);
            (parents[i])->setdelta(the_delta);
      break;
    default: fprintf(stderr, "Wrong kind of mutation\n");
    }
    parents[i]->computefitness();
  }

  
  
}

es::~es() {

}

void es::run() {
  FILE *fdesc_result;
  int tmpindex;
  int act_gen;
  double bestfitness;
  double worstfitness;
  double avgfitness;
  double tmpfitness;
  int i,j;
  ind *tmpind;
  

  /* Oeffne Ergebnisfile */
  fdesc_result = fopen("fitness.dat","w");

  /* Simulationslauf */
  for (act_gen=0; act_gen < the_generations; act_gen++) {

    /* schreibe Generationenzahl */
    fprintf(fdesc_result,"%d",act_gen);

    /* Berechne Fitness-Statistik */
    bestfitness = (parents[0])->getfitness();
    worstfitness = bestfitness;
    avgfitness = bestfitness;
    for (i=1; i<the_mu; i++) {
      tmpfitness = (parents[i])->getfitness();
      avgfitness += tmpfitness;
      if (tmpfitness < bestfitness) {
	bestfitness = tmpfitness;
      }
      if (tmpfitness > worstfitness) {
	worstfitness = tmpfitness;
      }
    }    
    avgfitness /= (1.0 * the_mu);

    fprintf(fdesc_result," %f %f %f\n", bestfitness, avgfitness, worstfitness);

    /* create children */
    for (i=0; i<the_lambda; i++) {
      tmpindex = randint(the_mu);
      if (tmpindex == the_mu)
	tmpindex = the_mu - 1;
      kids[i] = (parents[tmpindex])->clone();

      /* Mutation */
      preparemutation(kids[i]);
      (kids[i])->mutate();
      (kids[i])->computefitness();
      updateglobal((parents[tmpindex])->getfitness(), (kids[i])->getfitness());

    }

    /* Fitness sortieren */
    for (i=0; i<the_mu; i++) {
      for (j=the_lambda - 1; j > i; j--) {
	if ((kids[j])->getfitness() < (kids[j-1])->getfitness()) {
	  tmpind = kids[j];
	  kids[j] = kids[j-1];
	  kids[j-1] = tmpind;
	}
      }
    }

    

    /* selection */
    for (i=0; i<the_mu; i++) {
      tmpind = parents[i];
      parents[i] = kids[i];
      kids[i] = tmpind;
    } 

    /* delete offsprings */
    for (i=0; i<the_lambda; i++) {
      delete kids[i];
    }


  }

    /* schreibe Generationenzahl */
    fprintf(fdesc_result,"%d",act_gen);

  /* Berechne Fitness-Statistik */
  bestfitness = (parents[0])->getfitness();
  worstfitness = bestfitness;
  avgfitness = bestfitness;
  for (i=1; i<the_mu; i++) {
    tmpfitness = (parents[i])->getfitness();
    avgfitness += tmpfitness;
    if (tmpfitness < bestfitness) {
      bestfitness = tmpfitness;
    }
    if (tmpfitness > worstfitness) {
      worstfitness = tmpfitness;
    }
  }    
  avgfitness /= (1.0 * the_mu);
  
  fprintf(fdesc_result," %f %f %f\n", bestfitness, avgfitness, worstfitness);
  

  /* Dateien schliessen */
  fclose(fdesc_result);
}

void es::preparemutation(ind* individual) {

}

void es::updateglobal(double fit_parent, double fit_kid) {

}
