#include <stdio.h>
#include <math.h>
#include "fitnessfct.H"
#include "ackley_ff.H"
#include "sphere_ff.H"
#include "your_ff.H"
#include "myrandom.h"
#include "es.H"
#include "es_global.H"
#include "ind.H"
#include "ind_global.H"
#include "ind_uniform.H"
#include "ind_separate.H"

void printusage() 
{
  fprintf(stderr,"stdES <options>:\n");
  fprintf(stderr," -f <fitnessfunction> -- 0=ackley\n");
  fprintf(stderr,"                         1=sphere\n");
  fprintf(stderr,"                         2=your new ff\n");
  fprintf(stderr," -d <dimension>\n");
  fprintf(stderr," -l <lambda>\n");
  fprintf(stderr," -m <mu>\n");
  fprintf(stderr," -g <generations>\n");
  fprintf(stderr," -a <adaptation>  -- 0=1/5 rule\n");
  fprintf(stderr,"                     1=uniform selfadaptation\n");
  fprintf(stderr,"                     2=separate selfadaptation\n");
  fprintf(stderr," -D <initial delta>\n");
  fprintf(stderr," -s <seed>\n");
}

int main (int argc, char* argv[])
{
  int arg_index = 1;
  int tmp_fitnessarg = 0;
  int tmp_dimension = 30;
  int tmp_lambda = 100;
  int tmp_mu = 15;
  int tmp_generations = 100;
  int tmp_mutation = 0;
  long tmp_randomseed = 1;
  double tmp_initdelta = 1.0;

  es *es_instance;   /* Instanz der Evolutionsstrategie */

  double tmpfitness, oldfitness, bestfitness, meanfitness;
  ind *tmpind;
  int bestindex;
  double relprogress;
  double fitnessvar;
  int numberbetterkids;

  fitnessfct *thefitnessfct;



  int exitflag = 0;

  int i;


  if (argc > 1) {
    /* parsing */
    while (arg_index < argc) {
      if (strcmp(argv[arg_index], "-f") == 0) {
	arg_index++;
	tmp_fitnessarg = (int) strtol(argv[arg_index++],NULL,10);
      } else {
	if (strcmp(argv[arg_index], "-d") == 0) {
	  arg_index++;
	  tmp_dimension = (int) strtol(argv[arg_index++],NULL,10);
	} else {
	  if (strcmp(argv[arg_index], "-g") == 0) {
	    arg_index++;
	    tmp_generations = (int) strtol(argv[arg_index++],NULL,10);
	  } else {
	    if (strcmp(argv[arg_index], "-s") == 0) {
	      arg_index++;
	      tmp_randomseed = (long) strtol(argv[arg_index++],
					  NULL,10);
	    } else {
	      if (strcmp(argv[arg_index], "-a") == 0) {
		arg_index++;
		tmp_mutation = (int) strtol(argv[arg_index++],NULL,10);
	      } else {
		if (strcmp(argv[arg_index], "-l") == 0) {
		  arg_index++;
		  tmp_lambda = (int) strtol(argv[arg_index++],NULL,10);
		} else {
		  if (strcmp(argv[arg_index], "-D") == 0) {
		    arg_index++;
		    tmp_initdelta = (double) strtod(argv[arg_index++],NULL);
		  } else {
		    if (strcmp(argv[arg_index], "-m") == 0) {
		      arg_index++;
		      tmp_mu = (int) strtol(argv[arg_index++],NULL,10);
		    } else {
		      arg_index++;
		      fprintf(stderr,"unknown option\n");
		      exitflag = 1;
		    }
		  }
		}
	      }
	    }
	  }
	}
      }
    }
  }

  /* Initialisierung */
  setrandomseed(tmp_randomseed);
  /* der Fitnessfunktion */
  switch (tmp_fitnessarg) {
  case 0: thefitnessfct = new ackley_ff(tmp_dimension); break;
  case 1: thefitnessfct = new sphere_ff(tmp_dimension); break;
  case 2: thefitnessfct = new your_ff(tmp_dimension); break;
  default: fprintf(stderr,"unknown argument for fitness function\n"); exitflag = 1;
  }

  /* der ES */
  switch (tmp_mutation) {
  case 0: es_instance = new es_global(tmp_dimension, thefitnessfct, tmp_mu,
				      tmp_lambda, tmp_initdelta, tmp_generations,
				      tmp_mutation); 
          break;
  case 1: es_instance = new es(tmp_dimension, thefitnessfct, tmp_mu,
				       tmp_lambda, tmp_initdelta, tmp_generations,
				       tmp_mutation); 
          break;
  case 2: es_instance = new es(tmp_dimension, thefitnessfct, tmp_mu,
					tmp_lambda, tmp_initdelta, tmp_generations,
					tmp_mutation); 
          break;
  default:fprintf(stderr,"unknown argument for adaptation\n"); exitflag = 1;
  }

  if (!exitflag) {
    es_instance->run();
  } else {
    printusage();
  }

}
